#include "Spieler.h"
#include <math.h>
#include <TastaturEreignis.h>
#include <Globals.h>

// Inhalt der Spieler Klasse aus Spieler.h
// Konstruktor
Spieler::Spieler( SpielerTyp typ, int mapBr, int mapH, RandomGenerator *zRand )
{
    if( typ == GEGNER )
    {
        xSpeed = ( ( zRand->rand() * 8001 ) / 1000.0 - 4.0 ) * 60;
        ySpeed = ( ( zRand->rand() * 8001 ) / 1000.0 - 4.0 ) * 60;
        durchlssig = 3;
        farbe = 0xFFFF0000;
    }
    if( typ == ZIEL )
    {
        xSpeed = ( ( zRand->rand() * 7001 ) / 1000.0 - 4.0 ) * 60;
        ySpeed = ( ( zRand->rand() * 7001 ) / 1000.0 - 4.0 ) * 60;
        durchlssig = 3;
        farbe = 0xFF00FFFF;
    }
    if( typ == SPIELER )
    {
        xSpeed = 0;
        ySpeed = 0;
        farbe = 0xFF00FF00;
        durchlssig = 0;
    }
    xPos = ( zRand->rand() * ( mapBr - 100 ) ) + 50;
    yPos = ( zRand->rand() * ( mapH - 100 ) ) + 50;
    ref = 1;
}

Spieler::Spieler( SpielerTyp typ, Datei *zD )
{
    if( typ == GEGNER )
        farbe = 0xFFFF0000;
    if( typ == ZIEL )
        farbe = 0xFF00FFFF;
    if( typ == SPIELER )
        farbe = 0xFF00FF00;
    zD->lese( (char*)&xPos, 8 );
    zD->lese( (char*)&yPos, 8 );
    zD->lese( (char*)&xSpeed, 8 );
    zD->lese( (char*)&ySpeed, 8 );
    zD->lese( (char*)&durchlssig, 8 );
    ref = 1;
}

// Destruktor
Spieler::~Spieler()
{

}

// nicht constant
void Spieler::tick( double z, int mapBr, int mapH, char tastenStnde )
{
    if( farbe == 0xFF00FF00 )
    {
        if( ( tastenStnde | 4 ) == tastenStnde )
            xPos -= 150 * z;
        if( ( tastenStnde | 2 ) == tastenStnde )
            xPos += 150 * z;
        if( ( tastenStnde | 1 ) == tastenStnde )
            yPos -= 150 * z;
        if( ( tastenStnde | 8 ) == tastenStnde )
            yPos += 150 * z;
    }
    if( durchlssig )
    {
        durchlssig -= z;
        if( durchlssig < 0 )
            durchlssig = 0;
    }
    xPos += xSpeed * z;
    yPos += ySpeed * z;
    if( xPos < 10 )
    {
        xPos = 10;
        xSpeed = -xSpeed;
    }
    else if( xPos > mapBr - 10 )
    {
        xPos = mapBr - 10;
        xSpeed = -xSpeed;
    }
    else if( yPos < 10 )
    {
        yPos = 10;
        ySpeed = -ySpeed;
    }
    else if( yPos > mapH - 10 )
    {
        yPos = mapH - 10;
        ySpeed = -ySpeed;
    }
}

void Spieler::render( Bild &zRObj )
{
    if( !durchlssig || (int)( durchlssig * 8 ) % 2 )
    {
        zRObj.drawKreis( (int)xPos, (int)yPos, 10, farbe );
        zRObj.drawKreis( (int)xPos - 3, (int)yPos - 3, 2, farbe );
        zRObj.drawKreis( (int)xPos + 3, (int)yPos - 3, 2, farbe );
        zRObj.drawLinieV( (int)xPos, (int)yPos - 1, 4, farbe );
        zRObj.drawLinieH( (int)xPos - 3, (int)yPos + 5, 6, farbe );
    }
}

// constant
bool Spieler::berhrt( double px, double py ) const
{
    return !durchlssig && sqrt( (double)( px - xPos ) * ( px - xPos ) + ( py - yPos ) * ( py - yPos ) ) <= 20;
}

int Spieler::getX() const
{
    return (int)xPos;
}

int Spieler::getY() const
{
    return (int)yPos;
}

void Spieler::save( Datei *zD ) const
{
    zD->schreibe( (char*)&xPos, 8 );
    zD->schreibe( (char*)&yPos, 8 );
    zD->schreibe( (char*)&xSpeed, 8 );
    zD->schreibe( (char*)&ySpeed, 8 );
    zD->schreibe( (char*)&durchlssig, 8 );
}

// Reference Counting
Spieler *Spieler::getThis()
{
    ref++;
    return this;
}

Spieler *Spieler::release()
{
    ref--;
    if( !ref )
        delete this;
    return 0;
}